<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use \Teknisa\Model\Traits\TQuery;

class EscopoFaturamentoFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function editEscopoFaturamento($params)
    {
        $sql = "UPDATE FCT_TIPOORCAMENTO
                   SET 
                        NMTIPOORCAMENTO     = :NMTIPOORCAMENTO,
                        IDTIPOORCA          = :IDTIPOORCA,
                        IDALTQTDMEDIDAPROP  = :IDALTQTDMEDIDAPROP,
                        IDALTVRTABRPECPROP  = :IDALTVRTABRPECPROP,
                        IDUTILIZAVARPRECO   = :IDUTILIZAVARPRECO,
                        IDFILTRASERVICO     = :IDFILTRASERVICO,   
                        IDATIVO             = :IDATIVO,
                        NRORG               = :NRORG,
                        NRORGULTATU         = :NRORG,
                        CDOPERULTATU        = :CDOPERULTATU,
                        DTULTATU            = SYSDATE
                WHERE NRTIPOORCAMENTO   = :NRTIPOORCAMENTO";

        $this->connection->executeStatement($sql, $params);
    } 

    public function saveAddEscopoFaturamento($params)
    {
        $sql = "INSERT INTO FCT_TIPOORCAMENTO
                        (NRTIPOORCAMENTO,
                        NMTIPOORCAMENTO,
                        IDGERACONTRATO,
                        IDATIVO,
                        IDTIPOORCA,
                        IDALTQTDMEDIDAPROP,
                        IDALTVRTABRPECPROP,
                        IDUTILIZAVARPRECO,
                        IDFILTRASERVICO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRTIPOORCAMENTO,
                        :NMTIPOORCAMENTO,
                        :IDGERACONTRATO,
                        :IDATIVO,
                        :IDTIPOORCA,
                        :IDALTQTDMEDIDAPROP,
                        :IDALTVRTABRPECPROP,
                        :IDUTILIZAVARPRECO,
                        :IDFILTRASERVICO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    } 

    public function deleteEscopoFaturamento($params)
    {
        $sql = "DELETE FROM FCT_TIPOORCAMENTO WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";
        $this->connection->executeStatement($sql, $params);
    }

    public function getTabelaPrecoEscopoFat($params)
    {
        $sql = "SELECT
                    TABPREC.NRTABPRECESCOPOFAT,
                    TABPREC.NRTIPOORCAMENTO,
                    TABPREC.NRMOEDA,
                    TABPREC.CDPRECBASEVEND,
                    TABPREC.CDPRECBASESERV,
                    TABPREC.NRORG,
                    TABPREC.DTINCLUSAO,
                    TABPREC.DTULTATU,
                    TABPREC.NRORGINCLUSAO,
                    TABPREC.CDOPERINCLUSAO,
                    TABPREC.NRORGULTATU,
                    TABPREC.CDOPERULTATU,
                    ESCOPO.NMTIPOORCAMENTO,
                    MOEDA.NMMOEDA,
                    MOEDA.IDPADRAO,
                    PRECBAS.DSPRECOBASE,
                    PRECBASERV.DSPRECBASESERV
                FROM
                        FCT_TABPRECESCOPOFAT TABPREC
                    INNER JOIN FCT_TIPOORCAMENTO ESCOPO ON TABPREC.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                    INNER JOIN FCT_MOEDA         MOEDA ON TABPREC.NRMOEDA = MOEDA.NRMOEDA
                    LEFT JOIN PRECOBASE         PRECBAS ON TABPREC.CDPRECBASEVEND = PRECBAS.CDPRECOBASE
                    LEFT JOIN FCT_PRECBASESERV  PRECBASERV ON TABPREC.CDPRECBASESERV = PRECBASERV.CDPRECBASESERV
                WHERE TABPREC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }
    
    public function getMoedaTabelaPrecoEscopoFat()
    {
        $sql = "SELECT * FROM FCT_MOEDA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getPrecBaseServTabelaPrecoEscopoFat()
    {
        $sql = "SELECT * FROM FCT_PRECBASESERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getPrecBaseTabelaPrecoEscopoFat()
    {
        $sql = "SELECT * FROM PRECOBASE";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getValidaMoeda($params)
    {
        $sql = "SELECT
                    TABPREC.NRTABPRECESCOPOFAT,
                    TABPREC.NRTIPOORCAMENTO,
                    TABPREC.NRMOEDA,
                    TABPREC.CDPRECBASEVEND,
                    TABPREC.CDPRECBASESERV,
                    TABPREC.NRORG,
                    TABPREC.DTINCLUSAO,
                    TABPREC.DTULTATU,
                    TABPREC.NRORGINCLUSAO,
                    TABPREC.CDOPERINCLUSAO,
                    TABPREC.NRORGULTATU,
                    TABPREC.CDOPERULTATU,
                    ESCOPO.NMTIPOORCAMENTO,
                    MOEDA.NMMOEDA,
                    MOEDA.IDPADRAO,
                    PRECBAS.DSPRECOBASE,
                    PRECBASERV.DSPRECBASESERV
                FROM
                        FCT_TABPRECESCOPOFAT TABPREC
                    INNER JOIN FCT_TIPOORCAMENTO ESCOPO ON TABPREC.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                    INNER JOIN FCT_MOEDA         MOEDA ON TABPREC.NRMOEDA = MOEDA.NRMOEDA
                    LEFT JOIN PRECOBASE         PRECBAS ON TABPREC.CDPRECBASEVEND = PRECBAS.CDPRECOBASE
                    LEFT JOIN FCT_PRECBASESERV  PRECBASERV ON TABPREC.CDPRECBASESERV = PRECBASERV.CDPRECBASESERV
                WHERE
                        TABPREC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function editTabelaPreco($params)
    {
        $sql = "UPDATE FCT_TABPRECESCOPOFAT
                   SET 
                        NRMOEDA             = :NRMOEDA,
                        CDPRECBASESERV      = :CDPRECBASESERV,
                        CDPRECBASEVEND      = :CDPRECBASEVEND,
                        NRORGULTATU         = :NRORG,
                        CDOPERULTATU        = :CDOPERULTATU,
                        DTULTATU            = SYSDATE
                WHERE NRTABPRECESCOPOFAT   = :NRTABPRECESCOPOFAT";

        $this->connection->executeStatement($sql, $params);
    } 

    public function addTabelaPreco($params)
    {
        $sql = "INSERT INTO FCT_TABPRECESCOPOFAT
                       (NRTABPRECESCOPOFAT,
                        NRTIPOORCAMENTO,
                        NRMOEDA,
                        CDPRECBASEVEND,
                        CDPRECBASESERV,
                        NRORG,
                        IDATIVO,
                        NRORGINCLUSAO,
                        DTINCLUSAO,
                        CDOPERINCLUSAO)

                VALUES (:NRTABPRECESCOPOFAT,
                        :NRTIPOORCAMENTO,
                        :NRMOEDA,
                        :CDPRECBASEVEND,
                        :CDPRECBASESERV,
                        :NRORG,
                        :IDATIVO,
                        :NRORG,
                        SYSDATE,
                        :CDOPERINCLUSAO)";
        $this->connection->executeStatement($sql, $params);
    }

    public function deleteTabPrecEscopoFat($parametros)
    {
        $sql = "DELETE FROM FCT_TABPRECESCOPOFAT
                WHERE NRTABPRECESCOPOFAT IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function getItEscopoProposta($params)
    {
        $sql = "SELECT
                    ITM.NRITESCOPOPROPOSTA,
                    ITM.NRTIPOORCAMENTO,
                    ITM.CDSERVORCA,
                    ITM.CDTPMEDIDASERV,
                    ITM.NRORG,
                    ITM.IDATIVO,
                    ITM.DTINCLUSAO,
                    ITM.DTULTATU,
                    ITM.NRORGINCLUSAO,
                    ITM.CDOPERINCLUSAO,
                    ITM.NRORGULTATU,
                    ITM.CDOPERULTATU,
                    INC.NMOPERADOR AS NMOPERINCLUSAO,
                    ALT.NMOPERADOR AS NMOPERULTATU,
                    SERV.NMSERVORCA,
                    MED.DSTPMEDIDASERV,
                    ESCOPO.NMTIPOORCAMENTO,
                    ITM.IDPREVCOBRFUT
                FROM
                        FCT_ITESCOPOPROPOSTA ITM
                    INNER JOIN FCT_TIPOORCAMENTO  ESCOPO ON ITM.NRTIPOORCAMENTO = ESCOPO.NRTIPOORCAMENTO
                                                        AND ITM.NRORG = ESCOPO.NRORG
                    LEFT JOIN OPERADOR           INC ON ITM.CDOPERINCLUSAO = INC.CDOPERADOR
                                            AND ITM.NRORG = INC.NRORG
                    LEFT JOIN OPERADOR           ALT ON ITM.CDOPERULTATU = ALT.CDOPERADOR
                                            AND ITM.NRORG = ALT.NRORG
                    LEFT JOIN FCT_SERVICOORCA    SERV ON ITM.CDSERVORCA = SERV.CDSERVORCA
                                                    AND ITM.NRORG = SERV.NRORG
                    LEFT JOIN FCT_TIPOMEDIDASERV MED ON ITM.CDTPMEDIDASERV = MED.CDTPMEDIDASERV
                                                        AND ITM.NRORG = MED.NRORG
                WHERE ITM.NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getEspFatServicoOrca()
    {
        $sql = "SELECT
                    *
                FROM
                    FCT_SERVICOORCA
                WHERE
                        IDTIPO = 'S'
                    AND IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function validaServicoEscopoDuplicados($params)
    {
        $sql = "SELECT COUNT(*) AS VALIDASERV
                FROM FCT_ITESCOPOPROPOSTA
                WHERE CDSERVORCA = :CDSERVORCA
                AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                AND NRORG = :NRORG";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function insertEscFatServicoEscopo($params)
    {
        $sql = "INSERT INTO FCT_ITESCOPOPROPOSTA
                       (NRITESCOPOPROPOSTA,
                        NRTIPOORCAMENTO,
                        CDSERVORCA,
                        NRORG,
                        IDATIVO,
                        CDOPERINCLUSAO,
                        IDPREVCOBRFUT,
                        NRORGINCLUSAO,
                        DTINCLUSAO)
                VALUES (:NRITESCOPOPROPOSTA,
                        :NRTIPOORCAMENTO,
                        :CDSERVORCA,
                        :NRORG,
                        :IDATIVO,
                        :CDOPERINCLUSAO,
                        :IDPREVCOBRFUT,
                        :NRORG,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function editEscFatServicoEscopo($params)
    {
        $sql = "UPDATE FCT_ITESCOPOPROPOSTA
                   SET  IDPREVCOBRFUT = :IDPREVCOBRFUT,
                        NRORGULTATU = :NRORGULTATU,
                        CDOPERULTATU = :CDOPERULTATU,
                        DTULTATU  = SYSDATE
                 WHERE NRITESCOPOPROPOSTA  = :NRITESCOPOPROPOSTA";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteEscFatItServicoEscopo($params)
    {
        $sql = "DELETE FROM FCT_ITESCOPOPROPOSTA
                WHERE NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA";
        
        $this->connection->executeStatement($sql, $parametros, $params);
    }

    public function validaServEscParamFatDuplicados($params)
    {
        $sql = "SELECT COUNT(*) AS QTD
                FROM FCT_PARAMFATITESC
                WHERE NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA
                AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    } 
    
    public function insertServEscTabParamFat($params)
    {
        $sql = "INSERT INTO FCT_PARAMFATITESC
                        (NRPARAMFATITESC,
                        NRITESCOPOPROPOSTA,
                        NRTIPOORCAMENTO,
                        CDPRODFATURA,
                        CDFILIALFATURA,
                        NRPARCFATURA,
                        IDTPFATURA,
                        QTPARCELASFATU,
                        QTDIASAPURACAO,
                        NRDIACALCVENC,
                        QTDIASFATUVENC,
                        IDTPCALCVENC,
                        IDMESCALCVENC,
                        CDTIPOOPERFATURA,
                        CDSERIENF,
                        CDEMPRESA,
                        CDINSCESTA,
                        CDINSCMUNI,
                        CDTPMEDIDAVARIAVEL,
                        DSSERVORCABOL,
                        DSSERVORCANF,
                        QTPARCELASPARAUTO,
                        QTDIASINTERVALOPARAUTO,
                        NRDIAVENCIMENTOPARAUTO,
                        QTCARENCIADIASPARAUTO,
                        IDATIVO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRPARAMFATITESC,
                        :NRITESCOPOPROPOSTA,
                        :NRTIPOORCAMENTO,
                        :CDPRODFATURA,
                        :CDFILIALFATURA,
                        :NRPARCFATURA,
                        :IDTPFATURA,
                        :QTPARCELASFATU,
                        :QTDIASAPURACAO,
                        :NRDIACALCVENC,
                        :QTDIASFATUVENC,
                        :IDTPCALCVENC,
                        :IDMESCALCVENC,
                        :CDTIPOOPERFATURA,
                        :CDSERIENF,
                        :CDEMPRESA,
                        :CDINSCESTA,
                        :CDINSCMUNI,
                        :CDTPMEDIDAVARIAVEL,
                        :DSSERVORCABOL,
                        :DSSERVORCANF,
                        :QTPARCELASPARAUTO,
                        :QTDIASINTERVALOPARAUTO,
                        :NRDIAVENCIMENTOPARAUTO,
                        :QTCARENCIADIASPARAUTO,
                        :IDATIVO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    } 

    public function updateServEscTabParamFat($params)
    {
        $sql = "UPDATE FCT_PARAMFATITESC
                   SET 
                    NRITESCOPOPROPOSTA      = :NRITESCOPOPROPOSTA,
                    NRTIPOORCAMENTO         = :NRTIPOORCAMENTO,
                    CDPRODFATURA            = :CDPRODFATURA,
                    CDFILIALFATURA          = :CDFILIALFATURA,
                    NRPARCFATURA            = :NRPARCFATURA,
                    IDTPFATURA              = :IDTPFATURA,
                    QTPARCELASFATU          = :QTPARCELASFATU,
                    QTDIASAPURACAO          = :QTDIASAPURACAO,
                    NRDIACALCVENC           = :NRDIACALCVENC,
                    QTDIASFATUVENC          = :QTDIASFATUVENC,
                    IDTPCALCVENC            = :IDTPCALCVENC,
                    IDMESCALCVENC           = :IDMESCALCVENC,
                    CDTIPOOPERFATURA        = :CDTIPOOPERFATURA,
                    CDSERIENF               = :CDSERIENF,
                    CDEMPRESA               = :CDEMPRESA,
                    CDINSCESTA              = :CDINSCESTA,
                    CDINSCMUNI              = :CDINSCMUNI,
                    CDTPMEDIDAVARIAVEL      = :CDTPMEDIDAVARIAVEL,
                    DSSERVORCABOL           = :DSSERVORCABOL,
                    DSSERVORCANF            = :DSSERVORCANF,
                    QTPARCELASPARAUTO       = :QTPARCELASPARAUTO,
                    QTDIASINTERVALOPARAUTO  = :QTDIASINTERVALOPARAUTO,
                    NRDIAVENCIMENTOPARAUTO  = :NRDIAVENCIMENTOPARAUTO,
                    QTCARENCIADIASPARAUTO   = :QTCARENCIADIASPARAUTO,
                    IDATIVO                 = :IDATIVO,
                    NRORG                   = :NRORG,
                    NRORGULTATU             = :NRORG,
                    CDOPERULTATU            = :CDOPERULTATU,
                    DTULTATU                = SYSDATE 
                 WHERE NRPARAMFATITESC  = :NRPARAMFATITESC";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteServEscTabParamFat($params)
    {
        $sql = "DELETE FROM FCT_PARAMFATITESC WHERE NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA";
        $this->connection->executeStatement($sql, $params);
    }

    public function getTipoMedidaVariavelParamFat()
    {
        $sql = "SELECT
                    CDTPMEDIDASERV AS CDTPMEDIDAVARIAVEL,
                    DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                    NRORG,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU,
                    IDATIVO,
                    IDTPMEDIDASERV
                FROM
                    FCT_TIPOMEDIDASERV
                WHERE
                    IDATIVO = 'S'
                ORDER BY
                    DSTPMEDIDASERV ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Retorna os Parametros de Faturamento de um Item do serviço de Escopo
    public function getParamFatItEsc($params)
    {
        $sql = "SELECT
                    PARAM.NRPARAMFATITESC,
                    PARAM.NRITESCOPOPROPOSTA,
                    PARAM.DSSERVORCANF,
                    PARAM.DSSERVORCABOL,
                    PARAM.NRTIPOORCAMENTO,
                    PARAM.NRGRUPOFATURA,
                    PARAM.CDPRODFATURA,
                    PRODUTO.NMPRODUTO,
                    PARAM.CDFILIALFATURA,
                    FILIAL.NMFILIAL,
                    PARAM.NRPARCFATURA,
                    PARC.NMPRINCIPALPARC,
                    PARAM.IDTPFATURA,
                    PARAM.QTPARCELASFATU,
                    PARAM.IDATIVO,
                    PARAM.NRORG,
                    PARAM.DTINCLUSAO,
                    PARAM.DTULTATU,
                    PARAM.NRORGINCLUSAO,
                    PARAM.CDOPERINCLUSAO,
                    PARAM.NRORGULTATU,
                    PARAM.CDOPERULTATU,
                    PARAM.QTDIASAPURACAO,
                    PARAM.IDTPCALCVENC,
                    PARAM.NRDIACALCVENC,
                    PARAM.IDMESCALCVENC,
                    PARAM.QTDIASFATUVENC,
                    PARAM.CDTIPOOPERFATURA,
                    TPOPER.NMTIPOOPER     AS NMTPOPERNFS,
                    PARAM.CDSERIENF,
                    SERIENF.DSSERIE,
                    PARAM.CDINSCESTA,
                    PARAM.CDINSCMUNI,
                    PARAM.CDEMPRESA,
                    PARAM.CDTPMEDIDAVARIAVEL,
                    MEDVAR.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                    PARAM.QTPARCELASPARAUTO,
                    PARAM.QTDIASINTERVALOPARAUTO,
                    PARAM.NRDIAVENCIMENTOPARAUTO,
                    PARAM.QTCARENCIADIASPARAUTO
                FROM
                    FCT_PARAMFATITESC  PARAM
                    LEFT JOIN PRODUTO ON PARAM.CDPRODFATURA = PRODUTO.CDPRODUTO
                                        AND PARAM.NRORG = PRODUTO.NRORG
                    LEFT JOIN FILIAL ON PARAM.CDFILIALFATURA = FILIAL.CDFILIAL
                                        AND PARAM.NRORG = FILIAL.NRORG
                    LEFT JOIN PARCNEGOCIO        PARC ON PARAM.NRPARCFATURA = PARC.NRPARCNEGOCIO
                                                AND PARAM.NRORG = PARC.NRORG
                    LEFT JOIN TIPOOPERAC         TPOPER ON PARAM.CDTIPOOPERFATURA = TPOPER.CDTIPOOPER
                                                AND PARAM.NRORG = TPOPER.NRORG
                                                AND IDENTRSAIDOP = 'S'
                    LEFT JOIN SERIENF ON PARAM.CDSERIENF = SERIENF.CDSERIE
                                        AND PARAM.NRORG = SERIENF.NRORG
                                        AND PARAM.CDINSCESTA = SERIENF.CDINSCESTA
                                        AND PARAM.CDINSCMUNI = SERIENF.CDINSCMUNI
                    LEFT JOIN FCT_TIPOMEDIDASERV MEDVAR ON PARAM.CDTPMEDIDAVARIAVEL = MEDVAR.CDTPMEDIDASERV
                                                        AND PARAM.NRORG = MEDVAR.NRORG
                WHERE NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    // Dados relacionados as Competêcias de um item do Serviço de Escopo
    public function getParamfatitescserv($params)
    {
        $sql = "SELECT
                    NRPARAMFATITESCSERV,
                    NRITESCOPOPROPOSTA,
                    NRDIACOMPETINI,
                    NRDIACOMPETFIN,
                    IDMESCOMPETINI,
                    IDMESCOMPETFIN,
                    QTDIASAPURACAO,
                    IDTPCALCVENC,
                    NRDIACALCVENC,
                    IDMESCALCVENC,
                    QTDIASFATUVENC,
                    IDATIVO,
                    NRORG,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU
                FROM
                    FCT_PARAMFATITESCSERV
                WHERE
                    NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    } 

    public function validaServEscCompetenciaDuplicados($params)
    {
        $sql = "SELECT COUNT(*) AS QTD
                FROM FCT_PARAMFATITESCSERV
                WHERE NRITESCOPOPROPOSTA = :NRITESCOPOPROPOSTA";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    } 

    public function insertServEscTabCompetencia($params)
    {
        $sql = "INSERT INTO FCT_PARAMFATITESCSERV
                        ( NRPARAMFATITESCSERV,
                        NRITESCOPOPROPOSTA,
                        NRDIACOMPETINI,
                        IDMESCOMPETINI,
                        IDMESCOMPETFIN,
                        NRDIACOMPETFIN,
                        QTDIASAPURACAO,
                        IDTPCALCVENC,
                        NRDIACALCVENC,
                        IDMESCALCVENC,
                        QTDIASFATUVENC,
                        IDATIVO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRPARAMFATITESCSERV,
                        :NRITESCOPOPROPOSTA,
                        :NRDIACOMPETINI,
                        :IDMESCOMPETINI,
                        :IDMESCOMPETFIN,
                        :NRDIACOMPETFIN,
                        :QTDIASAPURACAO,
                        :IDTPCALCVENC,
                        :NRDIACALCVENC,
                        :IDMESCALCVENC,
                        :QTDIASFATUVENC,
                        :IDATIVO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";

        $this->connection->executeStatement($sql, $params);
    }

    public function updateServEscTabCompetencia($params)
    {
        $sql = "UPDATE FCT_PARAMFATITESCSERV
                   SET 
                    NRITESCOPOPROPOSTA      = :NRITESCOPOPROPOSTA,
                    NRDIACOMPETINI          = :NRDIACOMPETINI,
                    IDMESCOMPETINI          = :IDMESCOMPETINI,
                    IDMESCOMPETFIN          = :IDMESCOMPETFIN,
                    NRDIACOMPETFIN          = :NRDIACOMPETFIN,
                    QTDIASAPURACAO          = :QTDIASAPURACAO,
                    IDTPCALCVENC            = :IDTPCALCVENC,
                    NRDIACALCVENC           = :NRDIACALCVENC,
                    IDMESCALCVENC           = :IDMESCALCVENC,
                    QTDIASFATUVENC          = :QTDIASFATUVENC,
                    IDATIVO                 = :IDATIVO,
                    NRORG                   = :NRORG,
                    NRORGULTATU             = :NRORG,
                    CDOPERULTATU            = :CDOPERULTATU,
                    DTULTATU                = SYSDATE
                 WHERE NRPARAMFATITESCSERV  = :NRPARAMFATITESCSERV";

        $this->connection->executeStatement($sql, $params);
    } 

    public function deleteServEscTabCompetencia($params)
    {
        $sql = "DELETE FROM FCT_PARAMFATITESCSERV WHERE NRPARAMFATITESCSERV = :NRPARAMFATITESCSERV";
        $this->connection->executeStatement($sql, $params);
    }


    // Retorna os dados relacionados a TAB Dados Gerais dos Parametros de faturamento.
    public function getCapaParamFatEsc($params)
    {
        $sql = "SELECT
                    CAPA.NRCAPAPARAMFATESC,
                    CAPA.NRTIPOORCAMENTO,
                    CAPA.NRGRUPOFATURA,
                    CAPA.CDPRODFATURA,
                    PRODUTO.NMPRODUTO,
                    CAPA.CDFILIALFATURA,
                    FILIAL.NMFILIAL,
                    CAPA.NRPARCFATURA,
                    PARC.NMPRINCIPALPARC,
                    CAPA.IDTPFATURA,
                    CAPA.QTPARCELASFATU,
                    CAPA.IDTPCALCPARCINC,
                    CAPA.IDGERARATEIO,
                    CAPA.IDAPURAAPONTAMENTO,
                    CAPA.IDATIVO,
                    CAPA.NRORG,
                    CAPA.DTINCLUSAO,
                    CAPA.DTULTATU,
                    CAPA.NRORGINCLUSAO,
                    CAPA.CDOPERINCLUSAO,
                    CAPA.NRORGULTATU,
                    CAPA.CDOPERULTATU,
                    CAPA.QTDIASAPURACAO,
                    CAPA.IDTPCALCVENC,
                    CAPA.NRDIACALCVENC,
                    CAPA.IDMESCALCVENC,
                    CAPA.QTDIASFATUVENC,
                    CAPA.CDTIPOOPERFATURA,
                    TPOPER.NMTIPOOPER     AS NMTPOPERNFS,
                    CAPA.CDSERIENF,
                    SERIENF.DSSERIE,
                    CAPA.CDINSCESTA,
                    CAPA.CDINSCMUNI,
                    CAPA.CDEMPRESA,
                    CAPA.CDTPMEDIDAVARIAVEL,
                    MEDVAR.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                    IDTPCALCDTSAIDANF,
                    NRCALCULODTSAIDANF,
                    CAPA.DSSERVORCABOL,
                    CAPA.DSSERVORCANF,
                    NVL((
                        SELECT DISTINCT
                            'S'
                        FROM
                            FCT_RELACESCOPOAGRUPAMENTO
                        WHERE
                                NRTIPOORCAMENTO = CAPA.NRTIPOORCAMENTO
                            AND NRORG = CAPA.NRORG
                    ), 'N')               AS ESCOPOAGRUPADO,
                    IDAGRUPPARC,
                    IDMUNIPRESTSERV,
                    IDTIPONATUOPER,
                    IDNATUOPER,
                    CAPA.QTPARCELASPARAUTO,
                    CAPA.QTDIASINTERVALOPARAUTO,
                    CAPA.NRDIAVENCIMENTOPARAUTO,
                    CAPA.QTCARENCIADIASPARAUTO
                FROM
                    FCT_CAPAPARAMFATESC CAPA
                    LEFT JOIN PRODUTO ON CAPA.CDPRODFATURA = PRODUTO.CDPRODUTO
                                        AND CAPA.NRORG = PRODUTO.NRORG
                    LEFT JOIN FILIAL ON CAPA.CDFILIALFATURA = FILIAL.CDFILIAL
                                        AND CAPA.NRORG = FILIAL.NRORG
                    LEFT JOIN PARCNEGOCIO         PARC ON CAPA.NRPARCFATURA = PARC.NRPARCNEGOCIO
                                                AND CAPA.NRORG = PARC.NRORG
                    LEFT JOIN TIPOOPERAC          TPOPER ON CAPA.CDTIPOOPERFATURA = TPOPER.CDTIPOOPER
                                                AND CAPA.NRORG = TPOPER.NRORG
                                                AND IDENTRSAIDOP = 'S'
                    LEFT JOIN SERIENF ON CAPA.CDSERIENF = SERIENF.CDSERIE
                                        AND CAPA.NRORG = SERIENF.NRORG
                                        AND CAPA.CDINSCESTA = SERIENF.CDINSCESTA
                                        AND CAPA.CDINSCMUNI = SERIENF.CDINSCMUNI
                    LEFT JOIN FCT_TIPOMEDIDASERV  MEDVAR ON CAPA.CDTPMEDIDAVARIAVEL = MEDVAR.CDTPMEDIDASERV
                                                        AND CAPA.NRORG = MEDVAR.NRORG
                WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function validaCapaParamFatDuplicados($params)
    {
        $sql = "SELECT COUNT (*) AS QTD 
                FROM FCT_CAPAPARAMFATESC
                WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function insertCapaParametrosFaturamento($params)
    {
        $sql = "INSERT INTO FCT_CAPAPARAMFATESC
                        (NRCAPAPARAMFATESC,
                        NRTIPOORCAMENTO,
                        CDPRODFATURA,
                        CDFILIALFATURA,
                        NRPARCFATURA,
                        IDTPFATURA,
                        QTPARCELASFATU,
                        IDTPCALCPARCINC,
                        IDGERARATEIO,
                        IDAPURAAPONTAMENTO,
                        IDAGRUPPARC,
                        QTDIASAPURACAO,
                        NRDIACALCVENC,
                        QTDIASFATUVENC,
                        IDTPCALCVENC,
                        IDMESCALCVENC,
                        CDTIPOOPERFATURA,
                        CDEMPRESA,
                        CDSERIENF,
                        CDINSCESTA,
                        CDINSCMUNI,
                        IDTPCALCDTSAIDANF,
                        NRCALCULODTSAIDANF,
                        IDMUNIPRESTSERV,
                        IDTIPONATUOPER,
                        IDNATUOPER,
                        CDTPMEDIDAVARIAVEL,
                        DSSERVORCABOL,
                        DSSERVORCANF,
                        QTPARCELASPARAUTO,
                        QTDIASINTERVALOPARAUTO,
                        NRDIAVENCIMENTOPARAUTO,
                        QTCARENCIADIASPARAUTO,
                        IDATIVO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRCAPAPARAMFATESC,
                        :NRTIPOORCAMENTO,
                        :CDPRODFATURA,
                        :CDFILIALFATURA,
                        :NRPARCFATURA,
                        :IDTPFATURA,
                        :QTPARCELASFATU,
                        :IDTPCALCPARCINC,
                        :IDGERARATEIO,
                        :IDAPURAAPONTAMENTO,
                        :IDAGRUPPARC,
                        :QTDIASAPURACAO,
                        :NRDIACALCVENC,
                        :QTDIASFATUVENC,
                        :IDTPCALCVENC,
                        :IDMESCALCVENC,
                        :CDTIPOOPERFATURA,
                        :CDEMPRESA,
                        :CDSERIENF,
                        :CDINSCESTA,
                        :CDINSCMUNI,
                        :IDTPCALCDTSAIDANF,
                        :NRCALCULODTSAIDANF,
                        :IDMUNIPRESTSERV,
                        :IDTIPONATUOPER,
                        :IDNATUOPER,
                        :CDTPMEDIDAVARIAVEL,
                        :DSSERVORCABOL,
                        :DSSERVORCANF,
                        :QTPARCELASPARAUTO,
                        :QTDIASINTERVALOPARAUTO,
                        :NRDIAVENCIMENTOPARAUTO,
                        :QTCARENCIADIASPARAUTO,
                        :IDATIVO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function updateCapaParametrosFaturamento($params)
    {
        $sql = "UPDATE FCT_CAPAPARAMFATESC
                   SET
                    NRTIPOORCAMENTO         = :NRTIPOORCAMENTO,
                    CDPRODFATURA            = :CDPRODFATURA,   
                    CDFILIALFATURA          = :CDFILIALFATURA, 
                    NRPARCFATURA            = :NRPARCFATURA,   
                    IDTPFATURA              = :IDTPFATURA, 
                    QTPARCELASFATU          = :QTPARCELASFATU, 
                    IDTPCALCPARCINC         = :IDTPCALCPARCINC,
                    IDGERARATEIO            = :IDGERARATEIO,   
                    IDAPURAAPONTAMENTO      = :IDAPURAAPONTAMENTO, 
                    IDAGRUPPARC             = :IDAGRUPPARC,
                    QTDIASAPURACAO          = :QTDIASAPURACAO, 
                    NRDIACALCVENC           = :NRDIACALCVENC,  
                    QTDIASFATUVENC          = :QTDIASFATUVENC, 
                    IDTPCALCVENC            = :IDTPCALCVENC,   
                    IDMESCALCVENC           = :IDMESCALCVENC,  
                    CDTIPOOPERFATURA        = :CDTIPOOPERFATURA,   
                    CDEMPRESA               = :CDEMPRESA,  
                    CDSERIENF               = :CDSERIENF,  
                    CDINSCESTA              = :CDINSCESTA, 
                    CDINSCMUNI              = :CDINSCMUNI, 
                    IDTPCALCDTSAIDANF       = :IDTPCALCDTSAIDANF,  
                    NRCALCULODTSAIDANF      = :NRCALCULODTSAIDANF, 
                    IDMUNIPRESTSERV         = :IDMUNIPRESTSERV,
                    IDTIPONATUOPER          = :IDTIPONATUOPER, 
                    IDNATUOPER              = :IDNATUOPER, 
                    CDTPMEDIDAVARIAVEL      = :CDTPMEDIDAVARIAVEL, 
                    DSSERVORCABOL           = :DSSERVORCABOL,  
                    DSSERVORCANF            = :DSSERVORCANF,   
                    QTPARCELASPARAUTO       = :QTPARCELASPARAUTO,  
                    QTDIASINTERVALOPARAUTO  = :QTDIASINTERVALOPARAUTO, 
                    NRDIAVENCIMENTOPARAUTO  = :NRDIAVENCIMENTOPARAUTO, 
                    QTCARENCIADIASPARAUTO   = :QTCARENCIADIASPARAUTO,   
                    IDATIVO                 = :IDATIVO,
                    NRORG                   = :NRORG,
                    NRORGULTATU             = :NRORG,
                    CDOPERULTATU            = :CDOPERULTATU,
                    DTULTATU                = SYSDATE
                 WHERE NRCAPAPARAMFATESC  = :NRCAPAPARAMFATESC";

        $this->connection->executeStatement($sql, $params);
    } 
    
    public function deleteCapaParametrosFaturamento($params)
    {
        $sql = "DELETE FROM FCT_CAPAPARAMFATESC WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";
        $this->connection->executeStatement($sql, $params);
    }

    public function getProdutoParaFaturamentoParamFat()
    {
        $sql = "SELECT
                    CDPRODUTO,
                    CDARVPROD,
                    NMPRODUTO,
                    NRORG,
                    CDARVPROD || NMPRODUTO AS DESCRICAOPRODUTO
                FROM
                    PRODUTO
                WHERE
                    CDNVPRODUTO = (
                        SELECT
                            MAX(CDNVPRODUTO)
                        FROM
                            PRODUTO
                    )";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    } 

    public function getFilialParamFat()
    {
        $sql = "SELECT 
                    CDFILIAL,
                    NMFILIAL,
                    CDEMPRESA,
                    CDSERIE,
                    CDFILIAL || NMFILIAL AS DESCRICAOFILIAL
                FROM FILIAL 
                ORDER BY CDFILIAL ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTipoOperacParamFat()
    {
        $sql = "SELECT 
                    CDTIPOOPER,
                    NMTIPOOPER
                FROM TIPOOPERAC 
                WHERE IDENTRSAIDOP = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getSerieNotaParamFat($params)
    {
        $sql = "SELECT
                    CDSERIE,
                    DSSERIE,
                    CDINSCESTA,
                    CDINSCMUNI,
                    CDEMPRESA || CDINSCESTA || CDINSCMUNI || CDSERIE AS NRSERIE
                FROM
                    SERIENF
                WHERE
                    CDEMPRESA = :CDEMPRESA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getParamfatesc($params)
    {
        $sql = "SELECT
                    NRPARAMFATESC,
                    NRTIPOORCAMENTO,
                    NRDIACOMPETINI,
                    NRDIACOMPETFIN,
                    IDMESCOMPETINI,
                    IDMESCOMPETFIN,
                    QTDIASAPURACAO,
                    IDTPCALCVENC,
                    NRDIACALCVENC,
                    IDMESCALCVENC,
                    QTDIASFATUVENC,
                    IDATIVO,
                    NRORG,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU,
                    NVL((
                        SELECT DISTINCT
                            'S'
                        FROM
                            FCT_RELACESCOPOAGRUPAMENTO
                        WHERE
                                NRTIPOORCAMENTO = FCT_PARAMFATESC.NRTIPOORCAMENTO
                            AND NRORG = FCT_PARAMFATESC.NRORG
                    ), 'N') AS ESCOPOAGRUPADO
                FROM
                    FCT_PARAMFATESC
                WHERE
                    NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function validaParamFatCompetenciaDuplicados($params)
    {
        $sql = "SELECT COUNT(*) AS QTD
                FROM FCT_PARAMFATESC
                WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    } 

    public function insertParamFatCompetencia($params)
    {
        $sql = "INSERT INTO FCT_PARAMFATESC
                        (NRPARAMFATESC,
                        NRTIPOORCAMENTO,
                        NRDIACOMPETINI,
                        IDMESCOMPETINI,
                        IDMESCOMPETFIN,
                        NRDIACOMPETFIN,
                        QTDIASAPURACAO,
                        IDTPCALCVENC,
                        NRDIACALCVENC,
                        IDMESCALCVENC,
                        QTDIASFATUVENC,
                        IDATIVO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRPARAMFATESC,
                        :NRTIPOORCAMENTO,
                        :NRDIACOMPETINI,
                        :IDMESCOMPETINI,
                        :IDMESCOMPETFIN,
                        :NRDIACOMPETFIN,
                        :QTDIASAPURACAO,
                        :IDTPCALCVENC,
                        :NRDIACALCVENC,
                        :IDMESCALCVENC,
                        :QTDIASFATUVENC,
                        :IDATIVO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    public function updateParamFatCompetencia($params)
    {
        $sql = "UPDATE FCT_PARAMFATESC
                   SET
                    NRPARAMFATESC       = :NRPARAMFATESC,
                    NRTIPOORCAMENTO     = :NRTIPOORCAMENTO,
                    NRDIACOMPETINI      = :NRDIACOMPETINI,
                    IDMESCOMPETINI      = :IDMESCOMPETINI,
                    IDMESCOMPETFIN      = :IDMESCOMPETFIN,
                    NRDIACOMPETFIN      = :NRDIACOMPETFIN,
                    QTDIASAPURACAO      = :QTDIASAPURACAO,
                    IDTPCALCVENC        = :IDTPCALCVENC,
                    NRDIACALCVENC       = :NRDIACALCVENC,
                    IDMESCALCVENC       = :IDMESCALCVENC,
                    QTDIASFATUVENC      = :QTDIASFATUVENC,  
                    IDATIVO             = :IDATIVO,
                    NRORG               = :NRORG,
                    NRORGULTATU         = :NRORG,
                    CDOPERULTATU        = :CDOPERULTATU,
                    DTULTATU            = SYSDATE
                 WHERE NRPARAMFATESC  = :NRPARAMFATESC";

        $this->connection->executeStatement($sql, $params);
    } 
    
    public function deleteParamFatCompetencia($params)
    {
        $sql = "DELETE FROM FCT_PARAMFATESC WHERE NRPARAMFATESC = :NRPARAMFATESC";
        $this->connection->executeStatement($sql, $params);
    }
 
    public function getIndicesPadrEscopo($params)
    {
        $sql = "SELECT
                    FCT_INDICESPADRESCOPO.NRINDICESPADRESCOPO,
                    FCT_INDICESPADRESCOPO.NRTIPOORCAMENTO,
                    FCT_INDICESPADRESCOPO.CDTPINDICE,
                    FCT_INDICESPADRESCOPO.IDPERVALOR,
                    FCT_INDICESPADRESCOPO.CDINDICE,
                    INDICE.NMINDICE,
                    FCT_INDICESPADRESCOPO.NRPERCENTINDICE,
                    FCT_INDICESPADRESCOPO.IDATIVO,
                    FCT_INDICESPADRESCOPO.NRORG,
                    FCT_INDICESPADRESCOPO.DTINCLUSAO,
                    FCT_INDICESPADRESCOPO.NRORGINCLUSAO,
                    FCT_INDICESPADRESCOPO.CDOPERINCLUSAO,
                    FCT_INDICESPADRESCOPO.DTULTATU,
                    FCT_INDICESPADRESCOPO.NRORGULTATU,
                    FCT_INDICESPADRESCOPO.CDOPERULTATU
                FROM
                        FCT_INDICESPADRESCOPO
                    JOIN INDICE ON INDICE.CDTPINDICE = FCT_INDICESPADRESCOPO.CDTPINDICE
                                AND INDICE.IDPERVALOR = FCT_INDICESPADRESCOPO.IDPERVALOR
                                AND INDICE.CDINDICE = FCT_INDICESPADRESCOPO.CDINDICE
                                AND INDICE.NRORG = FCT_INDICESPADRESCOPO.NRORG
                WHERE
                    NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function validaRegistrosDuplicados($params)
    {
        $sql = "SELECT COUNT (*) AS QTD 
                FROM FCT_INDICESPADRESCOPO
                WHERE CDINDICE      = :CDINDICE
                AND NRINDICESPADRESCOPO <> NVL(:NRINDICESPADRESCOPO, -1)
                AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function updateIndiceFinanceiroPadrEscopo($params)
    {
        $sql = "UPDATE FCT_INDICESPADRESCOPO
                   SET CDINDICE             = :CDINDICE,
                       CDTPINDICE           = :CDTPINDICE,
                       IDPERVALOR           = :IDPERVALOR, 
                       NRPERCENTINDICE      = :NRPERCENTINDICE,
                       NRTIPOORCAMENTO      = :NRTIPOORCAMENTO,
                       NRORG                = :NRORG,
                       DTULTATU             = SYSDATE,
                       CDOPERULTATU         = :CDOPERULTATU,
                       IDATIVO              = :IDATIVO
                 WHERE NRINDICESPADRESCOPO  = :NRINDICESPADRESCOPO";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertIndiceFinanceiroPadrEscopo($params)
    {
        $sql = "INSERT INTO FCT_INDICESPADRESCOPO(
                    NRINDICESPADRESCOPO, CDINDICE, CDTPINDICE, IDPERVALOR, NRPERCENTINDICE, NRTIPOORCAMENTO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, IDATIVO)
                VALUES(
                    :NRINDICESPADRESCOPO, :CDINDICE, :CDTPINDICE, :IDPERVALOR, :NRPERCENTINDICE, :NRTIPOORCAMENTO, :NRORG, SYSDATE, :CDOPERINCLUSAO, :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteParametrosIndicePadrEscopo($params)
    {
        $sql = "DELETE FROM FCT_INDICESPADRESCOPO WHERE NRINDICESPADRESCOPO = :NRINDICESPADRESCOPO";
        $this->connection->executeStatement($sql, $params);
    }

    public function getRelacAgrupTipoOrcaEscopo($params)
    {
        $sql = "SELECT
                    RELAC.NRRELACAGRUPTIPOORCA,
                    RELAC.NRAGRUPAMENTOTIPOORCA,
                    AGRUP.NMAGRUPAMENTOTIPOORCA,
                    RELAC.NRTIPOORCAMENTO,
                    RELAC.NRORG
                FROM
                        FCT_RELACAGRUPTIPOORCA RELAC
                    JOIN FCT_AGRUPAMENTOTIPOORCA AGRUP ON AGRUP.NRAGRUPAMENTOTIPOORCA = RELAC.NRAGRUPAMENTOTIPOORCA
                                                        AND AGRUP.NRORG = RELAC.NRORG
                WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function validaDescricaoEscopoDuplicados($params)
    {
        $sql = "SELECT
                    COUNT(*) AS QTD
                FROM
                    FCT_DESCAGRUPESCOPO
                WHERE
                        NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                    AND ( ( IDTPFATURA = :IDTPFATURA )
                        OR ( IDTPFATURA IS NULL ) )
                    AND ( ( CDFILIALFATURA = :CDFILIALFATURA )
                        OR ( CDFILIALFATURA IS NULL ) )
                    AND ( ( NRPARCFATURA = :NRPARCFATURA )
                        OR ( NRPARCFATURA IS NULL ) )
                    AND ( ( CDTIPOOPERFATURA = :CDTIPOOPERFATURA )
                        OR ( CDTIPOOPERFATURA IS NULL ) )
                    AND ( ( CDSERIENF = :CDSERIENF )
                        OR ( CDSERIENF IS NULL ) )
                    AND ( ( CDEMPRESA = :CDEMPRESA )
                        OR ( CDEMPRESA IS NULL ) )
                    AND ( ( CDINSCESTA = :CDINSCESTA )
                        OR ( CDINSCESTA IS NULL ) )
                    AND ( ( CDINSCMUNI = :CDINSCMUNI )
                        OR ( CDINSCMUNI IS NULL ) )";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function insertDescricaoEscopo($params)
    {
        $sql = "    INSERT INTO FCT_DESCAGRUPESCOPO (
                        NRDESCAGRUPESCOPO,
                        DSNOMEAGRUPAMENTO,
                        IDATIVO,
                        NRTIPOORCAMENTO,
                        IDTPFATURA,
                        CDFILIALFATURA,
                        NRPARCFATURA,
                        CDTIPOOPERFATURA,
                        CDSERIENF,
                        CDEMPRESA,
                        CDINSCESTA,
                        CDINSCMUNI,
                        NRORG,
                        DTINCLUSAO,
                        NRORGINCLUSAO,
                        CDOPERINCLUSAO,
                        IDPREVCOBRFUT
                    ) VALUES (
                        :NRDESCAGRUPESCOPO,
                        :DSNOMEAGRUPAMENTO,
                        :IDATIVO,
                        :NRTIPOORCAMENTO,
                        :IDTPFATURA,
                        :CDFILIALFATURA,
                        :NRPARCFATURA,
                        :CDTIPOOPERFATURA,
                        :CDSERIENF,
                        :CDEMPRESA,
                        :CDINSCESTA,
                        :CDINSCMUNI,
                        :NRORG,
                        SYSDATE,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :IDPREVCOBRFUT
                    )";

        $this->connection->executeStatement($sql, $params);
    }

    public function updateDescricaoEscopo($params)
    {
        $sql = "UPDATE FCT_DESCAGRUPESCOPO
                   SET
                        DSNOMEAGRUPAMENTO   = :DSNOMEAGRUPAMENTO,
                        IDATIVO             = :IDATIVO,
                        IDTPFATURA          = :IDTPFATURA,
                        CDFILIALFATURA      = :CDFILIALFATURA,
                        NRPARCFATURA        = :NRPARCFATURA,
                        CDTIPOOPERFATURA    = :CDTIPOOPERFATURA,
                        CDSERIENF           = :CDSERIENF,
                        CDEMPRESA           = :CDEMPRESA,
                        CDINSCESTA          = :CDINSCESTA,
                        CDINSCMUNI          = :CDINSCMUNI,
                        NRORG               = :NRORG,
                        IDPREVCOBRFUT       = :IDPREVCOBRFUT,
                        NRORGULTATU         = :NRORG,
                        CDOPERULTATU        = :CDOPERULTATU,
                        DTULTATU            = SYSDATE
                WHERE NRDESCAGRUPESCOPO   = :NRDESCAGRUPESCOPO";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteEscFatDescricaoEscopo($params)
    {
        $sql = "DELETE FROM FCT_DESCAGRUPESCOPO WHERE NRDESCAGRUPESCOPO = :NRDESCAGRUPESCOPO";
        $this->connection->executeStatement($sql, $params);
    }

    public function getDescAgrupEscopo($params)
    {
        $sql = "SELECT
                    DES.NRDESCAGRUPESCOPO,
                    DES.DSNOMEAGRUPAMENTO,
                    DES.NRTIPOORCAMENTO,
                    DES.IDTPFATURA,
                    DES.CDFILIALFATURA,
                    FILIAL.NMFILIAL       AS NMFILIALFATURA,
                    DES.NRPARCFATURA,
                    PARC.NMPRINCIPALPARC  AS NMPARCFATURA,
                    DES.CDTIPOOPERFATURA,
                    TIPOOPERAC.NMTIPOOPER AS NMTIPOOPERFATURA,
                    DES.CDSERIENF,
                    SERIE.DSSERIE         AS DSSERIENF,
                    DES.CDEMPRESA,
                    DES.CDINSCESTA,
                    DES.CDINSCMUNI,
                    DES.NRORG,
                    DES.IDATIVO,
                    DES.DTINCLUSAO,
                    DES.DTULTATU,
                    DES.NRORGINCLUSAO,
                    DES.CDOPERINCLUSAO,
                    DES.NRORGULTATU,
                    DES.CDOPERULTATU,
                    DES.IDPREVCOBRFUT,
                    ESCP.NMTIPOORCAMENTO
                FROM
                        FCT_DESCAGRUPESCOPO DES
                    INNER JOIN FCT_TIPOORCAMENTO ESCP ON DES.NRTIPOORCAMENTO = ESCP.NRTIPOORCAMENTO
                                                        AND DES.NRORG = ESCP.NRORG
                    LEFT JOIN FILIAL ON DES.CDFILIALFATURA = FILIAL.CDFILIAL
                                        AND DES.NRORG = FILIAL.NRORG
                    LEFT JOIN PARCNEGOCIO       PARC ON DES.NRPARCFATURA = PARC.NRPARCNEGOCIO
                                                AND DES.NRORG = PARC.NRORG
                    LEFT JOIN TIPOOPERAC ON DES.CDTIPOOPERFATURA = TIPOOPERAC.CDTIPOOPER
                                            AND DES.NRORG = TIPOOPERAC.NRORG
                                            AND TIPOOPERAC.IDENTRSAIDOP = 'S'
                    LEFT JOIN SERIENF           SERIE ON DES.CDSERIENF = SERIE.CDSERIE
                                            AND DES.CDEMPRESA = SERIE.CDEMPRESA
                                            AND DES.CDINSCESTA = SERIE.CDINSCESTA
                                            AND DES.CDINSCMUNI = SERIE.CDINSCMUNI
                                            AND DES.NRORG = SERIE.NRORG
                WHERE
                    DES.NRTIPOORCAMENTO = :NRTIPOORCAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }
}
