<?php

namespace SM\Service;

use SM\Factory\Sm00004006DocumentoContratoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Service\FileServerService;
use SM\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Sm00004006DocumentoContratoService
{
    protected $documentoContratoFactory;
    protected $Environment;
    protected $newCode;
    protected $fileServerService;

    public function __construct(EntityManager $entityManager, Sm00004006DocumentoContratoFactory $documentoContratoFactory, Environment $Environment, NewCode $newCode, FileServerService $fileServerService, $fileServerUrl)
    {
        $this->documentoContratoFactory = $documentoContratoFactory;
        $this->connection             = $entityManager->getConnection();
        $this->Environment            = $Environment;
        $this->newCode                = $newCode;
        $this->fileServerService      = $fileServerService;
        $this->fileServerUrl          = $fileServerUrl;
    }

    public function getDocumentoContrato()
    {
        return $this->documentoContratoFactory->getDocumentoContrato();
    }

    public function getTabAnexoDocumentoContrato($params)
    {
        $paramsaux = array(
            "NRSEQCONTDOC"   => $params['NRSEQCONTDOC']
        );
        return $this->documentoContratoFactory->getTabAnexoDocumentoContrato($paramsaux);
    }

    public function getContratosDocumento($parametro)
    {
        return $this->documentoContratoFactory->getContratosDocumento($parametro);
    }

    public function inserirDocumentoContrato($parametros)
    {
        $this->connection->beginTransaction();
        $cdoperador = $this->Environment->getCdOperador();
        $nrorg = $this->Environment->getNrOrg();
        try {
            $nrseqmoddocfct = $this->newCode->getCodeByTableName("FCT_MODELODOC", $nrorg, 1, 10, true);

            $paramsModeloDocumento = array(
                "NRSEQMODDOCFCT"      => $nrseqmoddocfct,
                "DSMODELODOCORCA"     => $parametros["DSMODELODOCORCA"],
                "IDATIVO"             => "S"
            );
            
            $this->documentoContratoFactory->inserirModeloDocumento($paramsModeloDocumento);

            $nrseqcontdoc = $this->newCode->getCodeByTableName("NRSEQCONTDOC", $nrorg, 1, 10, true);
            $paramsDocumentoContrato = array(
                "NRSEQCONTDOC"      => $nrseqcontdoc,
                "NRSEQCONTRATO"     => $parametros["NRSEQCONTRATO"],
                "IDPADRAO"          => $parametros["IDPADRAO"],
                "VRTAMANHOLOGO"     => $parametros["VRTAMANHOLOGO"],
                "IDATIVO"           => "S",
                "CDOPERINCLUSAO"    => $cdoperador,
                "NRORG"             => $nrorg,
                "NRSEQMODDOCFCT"    => $nrseqmoddocfct
            );

            $this->documentoContratoFactory->inserirDocumentoContrato($paramsDocumentoContrato);

            $nrcontratodocsitu = $this->newCode->getCodeByTableName("FCT_CONTRATODOCSITU", $nrorg, 1, 10, true);
            $situacoes = implode(",", $parametros["IDSITUACAOCONTRATO"]);

            $paramsSituacaoContrato = array(
                "NRCONTRATODOCSITU"   => $nrcontratodocsitu,
                "IDSITUACAOCONTRATO"  => $situacoes,
                "IDATIVO"             => "S",
                "NRSEQCONTDOC"        => $nrseqcontdoc,
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador
            );
            $this->documentoContratoFactory->inserirSituacaoDocumentoContrato($paramsSituacaoContrato);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function editarDocumentoContrato($parametros)
    {
        $this->connection->beginTransaction();
        $cdoperador = $this->Environment->getCdOperador();

        $paramsEditDocCont = array(
            "IDPADRAO"      => $parametros["IDPADRAO"],
            "VRTAMANHOLOGO" => $parametros["VRTAMANHOLOGO"],
            "CDOPERULTATU"  => $cdoperador,
            "NRSEQCONTDOC"  => $parametros["NRSEQCONTDOC"]
        );

        $this->documentoContratoFactory->updateDocumentoContrato($paramsEditDocCont);

        $this->connection->commit();
    }

    public function deleteDocumentoContrato($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $documentoContrato) {
                $dataParam = array("NRSEQCONTDOC" => $documentoContrato["NRSEQCONTDOC"],);
                $this->documentoContratoFactory->deleteDocumentoContrato($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirAnexoDocumentoContrato($parametros)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        $this->connection->beginTransaction();

        $paramsauxdoc = array(
            "NRSEQCONTDOC" => $parametros['NRSEQCONTDOC'],
        );

        $nmarquivo = $this->documentoContratoFactory->getDocumentoAnt($paramsauxdoc);

        $paramsauxdocidioma = array(
            "NRSEQCONTDOC"        => $parametros['NRSEQCONTDOC'],
            "CDIDIOM"             => $parametros['CDIDIOM'],
            "NRCONTRATODOCIDIOMA" => $parametros['NRCONTRATODOCIDIOMA'],
        );

        $valor = $this->documentoContratoFactory->validaDocIdioma($paramsauxdocidioma);

        if ($valor[0]["QTD"] > 0 || ($parametros["IDPADRAO"] == "S" && $nmarquivo[0]["NMARQUIVO"] != null && $parametros["IDPADRAO_OLD"] != "S")) {
            throw new \Exception("EXISTE");
        }

        try {
            if (count($parametros['ARQUIVO']) > 0) {
                $dados = $this->fileServerService->uploadFiles($parametros['ARQUIVO'], 'anexo_modelo_documento_contrato');
                $dados[0]["size"] = $dados[0]["size"] / 1024;
            } else {
                $dados = [[
                    "path" => $parametros["DSCAMINHOARQ"],
                    "size" => $parametros["VRTAMANHO"],
                    "name" => $parametros["NMARQUIVO"],
                    "type" => $parametros["DSTIPO"],
                ]];
            }

            if ($parametros["IDPADRAO"] == "S") {
                $params = array(
                    "NRSEQCONTDOC"    => $parametros['NRSEQCONTDOC'],
                    "NMARQUIVO"         => $dados[0]["name"],
                    "DSCAMINHOARQ"      => $dados[0]["path"],
                    "DSTIPO"            => $dados[0]["type"],
                    "VRTAMANHO"         => $dados[0]["size"],
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador
                );

                $this->documentoContratoFactory->atualizarModeloDocumentoContrato($params);
            }

            if ($parametros['NRCONTRATODOCIDIOMA'] == null) {
                $nrcontratodocidioma = $this->newCode->getCodeByTableName("FCT_CONTRATODOCIDIOMA", $nrorg, 1, 10, true);
                $params = array(
                    "NRCONTRATODOCIDIOMA" => $nrcontratodocidioma,
                    "NRSEQCONTDOC"    => $parametros['NRSEQCONTDOC'],
                    "CDIDIOM"           => $parametros['CDIDIOM'],
                    "IDATIVO"           => $parametros['IDATIVO'],
                    "NMARQUIVO"         => $dados[0]['name'],
                    "DSCAMINHOARQ"      => $dados[0]['path'],
                    "DSTIPO"            => $dados[0]['type'],
                    "VRTAMANHO"         => $dados[0]['size'],
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                );

                $this->documentoContratoFactory->inserirModeloDocumentoContrato($params);

                if ($parametros["IDPADRAO"] == "N" && $parametros["IDPADRAO_OLD"] == "S") {
                    $params = array(
                        "NRSEQCONTDOC"    => $parametros['NRSEQCONTDOC'],
                    );

                    $this->documentoContratoFactory->limparAnexoModeloDocContrato($params);
                }
            } else {
                $params = array(
                    "NRCONTRATODOCIDIOMA" => $parametros['NRCONTRATODOCIDIOMA'],
                    "NRSEQCONTDOC"    => $parametros['NRSEQCONTDOC'],
                    "CDIDIOM"           => $parametros['CDIDIOM'],
                    "IDATIVO"           => $parametros['IDATIVO'],
                    "NMARQUIVO"         => $dados[0]["name"],
                    "DSCAMINHOARQ"      => $dados[0]["path"],
                    "DSTIPO"            => $dados[0]["type"],
                    "VRTAMANHO"         => $dados[0]["size"],
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador
                );

                $this->documentoContratoFactory->updateModeloDocumentoContrato($params);

                if ($parametros["IDPADRAO"] == "N" && $parametros["IDPADRAO_OLD"] == "S") {
                    $params = array(
                        "NRSEQCONTDOC"    => $parametros['NRSEQCONTDOC'],
                    );

                    $this->documentoContratoFactory->limparAnexoModeloDocContrato($params);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function downloadAnexoModeloDocumentoContrato($idpadrao, $nrcontratodocidioma, $nrseqcontdoc)
    {
        if ($idpadrao == "S") {
            $param = array(
                "NRSEQCONTDOC" => $nrseqcontdoc
            );
            $dados = $this->documentoContratoFactory->getDocumentoAnt($param);
        } else {
            $param = array(
                "NRCONTRATODOCIDIOMA" => $nrcontratodocidioma
            );
            $dados = $this->documentoContratoFactory->getDocumento($param);
        }

        $url = $this->fileServerUrl . "/files/" . $dados[0]['DSCAMINHOARQ'];
        $nomeArquivo = $dados[0]['NMARQUIVO'];
        $tipo = $dados[0]['DSTIPO'];
        $tamanho = $dados[0]['VRTAMANHO'] * 1024;

        $file = @file_get_contents($url);
        if ($file) {
            Utilities::downloadArquivoFileServer($url, $nomeArquivo, $tipo, $tamanho);
        } else {
            echo "Arquivo não encontrado.";
            exit;
        }
    }

    public function limparAnexoModeloDocumentoContrato($parametros)
    {
        $row = $parametros["row"];

        $param = array(
            "NRCONTRATODOCIDIOMA" => $row["NRCONTRATODOCIDIOMA"]
        );

        $this->fileServerService->deleteFile($row["DSCAMINHOARQ"]);
        $this->documentoContratoFactory->limparAnexoModeloDocumentoContrato($param);

        if ($row["IDPADRAO"] == "S") {
            $param = array(
                "NRSEQCONTDOC" => $row["NRSEQCONTDOC"]
            );
            $this->documentoContratoFactory->limparAnexoModeloDocContrato($param);
        }
    }

    public function deletarAnexoDocumentoContrato($params)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $anexo) {
                $dataParam = array(
                    "NRCONTRATODOCIDIOMA" => $anexo["NRCONTRATODOCIDIOMA"],
                );
                if (!empty($anexo['DSCAMINHOARQ'])) {
                    $this->fileServerService->deleteFile($anexo['DSCAMINHOARQ']);
                }
                if ($dataParam["NRCONTRATODOCIDIOMA"] !== null) {
                    $this->documentoContratoFactory->deletarAnexoDocumentoContrato($dataParam);
                }
                if ($anexo["IDPADRAO"] == "S") {
                    $param = array(
                        "NRSEQCONTDOC" => $anexo['NRSEQCONTDOC']
                    );
                    $this->documentoContratoFactory->limparAnexoModeloDocContrato($param);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizarAnexoModeloDocumentoContrato($params)
    {
        $nrorg = $this->Environment->getNrOrg();
        $cdoperador = $this->Environment->getCdOperador();
        $this->connection->beginTransaction();
        try {
            $paramsAuxDocContIdi = [
                "NRCONTRATODOCIDIOMA" => $params["row"]["NRCONTRATODOCIDIOMA"],
                "NRORG" => $nrorg
            ];
            $datasetDocContIdi = $this->documentoContratoFactory->getDocumentoContratoIdioma($paramsAuxDocContIdi);
            $doc = $datasetDocContIdi[0];

            $paramsAuxModDoc = [
                "NRSEQMODDOCFCT" => $doc["NRSEQMODDOCFCT"],
                "CDIDIOM"        => $doc["CDIDIOM"]
            ];
            $datasetModDoc = $this->documentoContratoFactory->getModeloDocumentoIdioma($paramsAuxModDoc);
            if (count($datasetModDoc) > 0) {
                $modeloDoc = $datasetModDoc[0];

                if (!empty($doc["DSCAMINHOARQ"])) {
                    $this->fileServerService->deleteFile($doc["DSCAMINHOARQ"]);
                }

                $nomeArquivo  = $modeloDoc["NMARQUIVO"];
                $tipo         = $modeloDoc["DSTIPO"];
                $tamanhoBytes = $modeloDoc["VRTAMANHO"] * 1024;
                $path         = $modeloDoc["DSCAMINHOARQ"];

                if (!empty($path)) {
                    $url = $this->fileServerUrl . "/files/" . $path;
                    $conteudo = $this->fileServerService->getDocumentoFileServer($url, $nrorg);
                    if ($conteudo !== false) {
                        $file = $this->fileServerService->uploadFileServer('anexo_modelo_documento_contrato', $nomeArquivo, $conteudo, $tamanhoBytes, $tipo);
                        $caminhoNovo = $file["path"];

                        $this->atualizarDocumentoContratoIdioma($params["row"]["NRCONTRATODOCIDIOMA"], $nrorg, $cdoperador, $nomeArquivo, $tipo, $tamanhoBytes, $caminhoNovo);

                        if ($params["row"]["IDPADRAO"] == "S") {
                            $this->atualizarAnexoDocumentoContrato($params["row"]["NRSEQCONTDOC"], $nomeArquivo, $tamanhoBytes, $tipo, $caminhoNovo);
                        }
                    }
                }
            } else {
                throw new \Exception("SEM_MODELO");
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function atualizarDocumentoContratoIdioma($nrcontratodocidioma, $nrorg, $cdoperador, $nomeArquivo = null, $tipo = null, $tamanhoBytes = null, $path = null)
    {
        $params = [
            "NMARQUIVO"            => $nomeArquivo,
            "DSCAMINHOARQ"         => $path,
            "DSTIPO"               => $tipo,
            "VRTAMANHO"            => round($tamanhoBytes / 1024, 2),
            "CDOPERULTATU"         => $cdoperador,
            "NRCONTRATODOCIDIOMA" => $nrcontratodocidioma,
            "NRORG"                => $nrorg
        ];

        $this->documentoContratoFactory->atualizarDocumentoContratoIdioma($params);
    }

    public function atualizarAnexoDocumentoContrato($nrseqcontdoc, $nmarquivo, $vrtamanho, $dstipo, $dscaminhoarq)
    {
        $params = [
            "NMARQUIVO"    => $nmarquivo,
            "VRTAMANHO"    => $vrtamanho,
            "DSTIPO"       => $dstipo,
            "DSCAMINHOARQ" => $dscaminhoarq,
            "NRSEQCONTDOC" => $nrseqcontdoc
        ];
        $this->documentoContratoFactory->atualizarAnexoDocumentoContrato($params);
    }

    public function getParceiroNegocioContrato()
    {
        return $this->documentoContratoFactory->getParceiroNegocioContrato();
    }

    public function getNomeContrato()
    {
        return $this->documentoContratoFactory->getNomeContrato();
    }
}
