<?php

namespace SM\Service;

use SM\Factory\Sm00070000HierarquiaOperadoresFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Sm00070000HierarquiaOperadoresService
{
    protected $connection;
    protected $hierarquiaOperadoresFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Sm00070000HierarquiaOperadoresFactory $hierarquiaOperadoresFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->hierarquiaOperadoresFactory  = $hierarquiaOperadoresFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getHierarquiaOperadores()
    {
        $data = $this->hierarquiaOperadoresFactory->getHierarquiaOperadores();
        $data["data"] = $this->fixParentsForTreeGrid($data["data"], "NRHIERARQUIAOPER", "NRHIERARQUIAPAI");
        return $data;
    }

     /**
     * Tree Grid has a problem: when he gets a data that has
     * an item with PARENT equals a value that does not exists
     * in Database, it just disappears.
     * So, this method is a little tool, setting non-existent
     * IDs as null, avoiding problem with Tree Grid.
     * @param array $values
     * @param string $indexPrimary
     * @param string $indexParent
     * @return mixed
     */
    public static function fixParentsForTreeGrid(array $values, $indexPrimary = 'ID', $indexParent = 'PARENT')
    {
        $aux = array();
        foreach ($values as $item) {
            array_push($aux, $item[$indexPrimary]);
        }
        foreach ($values as &$item) {
            if (!in_array($item[$indexParent], $aux)) {
                $item[$indexParent] = null;
            }
        }
        return $values;
    }

    public function insertHierarquia($params)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        try {
            if ($params["NRHIERARQUIAOPER"] == null) {
                $nrhierarquiaoper = $this->newCode->getCodeByTableName("CRM_HIERARQUIAOPER", $nrorg, 1, 10, true);
                $dataParamsHierarquia = array (
                    "NRHIERARQUIAOPER"  => $nrhierarquiaoper,
                    "DSHIERARQUIAOPER"  => $params["DSHIERARQUIAOPER"],
                    "NRHIERARQUIAPAI"   => $params["NRHIERARQUIAPAI"],
                    "IDOPERMASTER"      => $params["IDOPERMASTER"],
                    "NRORG"             => $nrorg,
                    "CDOPERADOR"        => $params["CDOPERADOR"],
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRORGINCLUSAO"     => $nrorg,
                    "IDATIVO"           => 'S',
                    "CDFILIAL"          => $params["CDFILIAL"]
                );
                $this->hierarquiaOperadoresFactory->insertHierarquia($dataParamsHierarquia);
            } else {
                $dataParamsHierarquia = array (
                    "DSHIERARQUIAOPER"  => $params["DSHIERARQUIAOPER"],
                    "NRHIERARQUIAPAI"   => $params["NRHIERARQUIAPAI"],
                    "IDOPERMASTER"      => $params["IDOPERMASTER"],
                    "NRORG"             => $nrorg,
                    "CDOPERADOR"        => $params["CDOPERADOR"],
                    "CDOPERULTATU"      => $cdoperador,
                    "NRORGULTATU"       => $nrorg,
                    "IDATIVO"           => 'S',
                    "CDFILIAL"          => $params["CDFILIAL"],
                    "NRHIERARQUIAOPER"  => $params["NRHIERARQUIAOPER"],
                );
                $this->hierarquiaOperadoresFactory->updateHierarquia($dataParamsHierarquia);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getOperadorHierarquia()
    {
        return $this->hierarquiaOperadoresFactory->getOperadorHierarquia();
    }

    public function deleteHierarquia($params)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $registros) {
                $dataParam = array( "NRHIERARQUIAOPER" => $registros["NRHIERARQUIAOPER"],);
                $this->hierarquiaOperadoresFactory->deleteHierarquia($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
